package main

import (
	"encoding/gob"
	"log"
	"net/http"
	"os"

	"streifling.com/jason/cpolis/cmd/control"
	"streifling.com/jason/cpolis/cmd/model"
	"streifling.com/jason/cpolis/cmd/view"
)

func init() {
	gob.Register(model.User{})
}

func main() {
	cliArgs, err := control.HandleCliArgs()
	if err != nil {
		log.Fatalln(err)
	}

	logFile, err := os.OpenFile(cliArgs.LogFile, os.O_WRONLY|os.O_CREATE|os.O_APPEND, 0644)
	if err != nil {
		log.Fatalln(err)
	}
	defer logFile.Close()
	log.SetOutput(logFile)

	db, err := model.OpenDB("cpolis")
	if err != nil {
		log.Fatalln(err)
	}
	defer db.Close()

	key, err := control.LoadKey(cliArgs.KeyFile)
	if err != nil {
		key, err = control.NewKey()
		if err != nil {
			log.Fatalln(err)
		}
		control.SaveKey(key, cliArgs.KeyFile)
	}
	store := control.NewCookieStore(key)

	mux := http.NewServeMux()
	mux.Handle("/web/static/", http.StripPrefix("/web/static/",
		http.FileServer(http.Dir(cliArgs.WebDir+"/static/"))))
	mux.HandleFunc("/", view.HomePage(cliArgs, db, store))

	mux.HandleFunc("GET /create-tag/", view.CreateTag(cliArgs))
	mux.HandleFunc("GET /create-user/", view.CreateUser(cliArgs))
	mux.HandleFunc("GET /edit-user/", view.EditUser(cliArgs, db, store))
	mux.HandleFunc("GET /hub/", view.ShowHub(cliArgs, db, store))
	mux.HandleFunc("GET /logout/", view.Logout(cliArgs, store))
	mux.HandleFunc("GET /publish-issue/", view.PublishLatestIssue(cliArgs, db, store))
	mux.HandleFunc("GET /rejected-articles/", view.ShowRejectedArticles(cliArgs, db, store))
	mux.HandleFunc("GET /rss/", view.ShowRSS(cliArgs,
		db,
		"Freimaurer Distrikt Niedersachsen und Sachsen-Anhalt",
		"https://distrikt-ni-st.de",
		"Freiheit, Gleichheit, Brüderlichkeit, Toleranz und Humanität",
	))
	mux.HandleFunc("GET /this-issue/", view.ShowCurrentArticles(cliArgs, db))
	mux.HandleFunc("GET /unpublished-articles/", view.ShowUnpublishedArticles(cliArgs, db))
	mux.HandleFunc("GET /write-article/", view.WriteArticle(cliArgs, db))

	mux.HandleFunc("POST /add-tag/", view.AddTag(cliArgs, db, store))
	mux.HandleFunc("POST /add-user/", view.AddUser(cliArgs, db, store))
	mux.HandleFunc("POST /login/", view.Login(cliArgs, db, store))
	mux.HandleFunc("POST /publish-article/", view.PublishArticle(cliArgs, db, store))
	mux.HandleFunc("POST /reject-article/", view.RejectArticle(cliArgs, db, store))
	mux.HandleFunc("POST /resubmit-article/", view.ResubmitArticle(cliArgs, db, store))
	mux.HandleFunc("POST /review-rejected-article/", view.ReviewRejectedArticle(cliArgs, db, store))
	mux.HandleFunc("POST /review-unpublished-article/", view.ReviewUnpublishedArticle(cliArgs, db, store))
	mux.HandleFunc("POST /submit-article/", view.SubmitArticle(cliArgs, db, store))
	mux.HandleFunc("POST /update-user/", view.UpdateUser(cliArgs, db, store))
	mux.HandleFunc("POST /upload-image/", view.UploadImage(cliArgs))

	log.Fatalln(http.ListenAndServe(":8080", mux))
}
